#define PLUGIN_VERSION		"1.1"

#pragma semicolon 1
#pragma newdecls required

#include <sourcemod>
#include <sdktools>

#define CVAR_FLAGS			FCVAR_NOTIFY

#define DEBUG 0

public Plugin myinfo =
{
	name = "[ANY] Custom Skybox (DD)",
	author = "Dragokas",
	description = "Set custom skybox (with delayed downloader support)",
	version = PLUGIN_VERSION,
	url = "http://github.com/dragokas"
}

/*
	ChangeLog:
	
	1.0 (01-Jan-2020)
	 - Concept release (technical demo)
	 
	1.1 (26-Mar-2020)
	 - Added support for all games.
	 - No need to define default skybox anymore. It is detected automatically.
	 - No need to list all default skyboxes. They are detected automatically.
	 
	 TODO:
	 - Transitional player is detected; timers are removed. How to send different cvar values for diff. users on very early stage ? Need testers.
*/

ConVar g_hCvarSkyName;
char g_sSkyBox[64], g_sSkyBoxDefault[64], g_sMap[64];
StringMap hMapSky;
ArrayList g_aSky;
int g_iSky = -1;
//bool g_bFirstConnect[MAXPLAYERS+1] = {true, ...};
//bool g_bTransitional[MAXPLAYERS+1];
//bool g_bOverride; //[MAXPLAYERS+1];

/*
public Action OnClientPreConnect(char[] name, const char[] password, const char[] ip, const char[] steamID, char rejectReason[255])
{
	PrintToServer("Name=%s, password=%s, IP=%s, steamID=%s", name, password, ip, steamID);
	FormatEx(rejectReason, sizeof(rejectReason), "%s", "#Valve_Reject_Server_Full");
	return Plugin_Stop;
}
*/

public void OnPluginStart()
{
	hMapSky = new StringMap();
	GetSkyBoxes();
	
	g_hCvarSkyName = FindConVar("sv_skyname");
	g_aSky = new ArrayList(ByteCountToCells(64));
	
	#if DEBUG
	RegAdminCmd("sm_sky", CmdSkyName, ADMFLAG_ROOT, "Show current sky name");
	RegAdminCmd("sm_over", CmdSkyOver, ADMFLAG_ROOT, "Show current sky name");
	RegAdminCmd("sm_info", CmdSkyInfo, ADMFLAG_ROOT, "Show current sky name");
	#endif
}

public Action CmdSkyName(int client, int args)
{
	if( g_sSkyBox[0] == 0 )
	{
		g_hCvarSkyName.GetString(g_sSkyBox, sizeof g_sSkyBox);
	}
	PrintToChat(client, "Current skybox name: \x05%s", g_sSkyBox);
	return Plugin_Handled;
}

public Action CmdSkyOver(int client, int args)
{
	SendConVarValue(client, g_hCvarSkyName, "test_moon_hdr");
	
	return Plugin_Handled;
}

public Action CmdSkyInfo(int client, int args)
{
	//PrintToChat(client, "Is transitional? %b", g_bTransitional[client]);
	return Plugin_Handled;
}

/*
public Action Event_PlayerDisconnect(Event event, const char[] name, bool dontBroadcast)
{
	int client = GetClientOfUserId(event.GetInt("userid"));
	g_bFirstConnect[client] = true;
	g_bTransitional[client] = false;
}

public void OnClientConnected(int client)
{
	if( g_bFirstConnect[client] )
	{
		g_bFirstConnect[client] = false;
		g_bTransitional[client] = false;
	}
	else {
		g_bTransitional[client] = true;
	}
}

void ApplyClientSky(int client)
{
	if( client && !IsFakeClient(client) )
	{
		if( g_bTransitional[client] )
		{
			SendConVarValue(client, g_hCvarSkyName, "test_moon_hdr");
		}
		else {
			SendConVarValue(client, g_hCvarSkyName, "sky_day01_09_hdr");
		}
	}
}

void ApplyClientSkyAll()
{
	for( int i = 1; i <= MaxClients; i++)
	{
		if( IsClientConnected(i) && !IsFakeClient(i) )
		{
			ApplyClientSky(i);
		}
	}
}
*/

/*
public void OnGameFrame()
{
	
	for( int i = 1; i <= MaxClients; i++)
	{
		if( g_bOverride[i] )
		{
			ApplyClientSky(i);
		}
	}
	
	if( g_bOverride )
	{
		ApplyClientSkyAll();
	}
}
*/

void LoadSky()
{
	g_aSky.Clear();
	
	g_aSky.PushString("underwater_corals");
	//g_aSky.PushString("bright_stars");
	//g_aSky.PushString("evening_lights");
	//g_aSky.PushString("inside_forest");
	//g_aSky.PushString("pink_nebula");
	//g_aSky.PushString("snow_birth_forest");
	//g_aSky.PushString("snow_bridge");
	//g_aSky.PushString("snow_pine_tree");
	//g_aSky.PushString("star_lights");
	//g_aSky.PushString("supernova_nebula");
	//g_aSky.PushString("white_sky");

}

public void OnMapStart()
{
	GetCurrentMap(g_sMap, sizeof g_sMap);

	g_hCvarSkyName.GetString(g_sSkyBoxDefault, sizeof g_sSkyBoxDefault);

	if( g_iSky != -1 )
	{
		g_aSky.Erase(g_iSky);
	}
	if( g_aSky.Length == 0 )
	{
		LoadSky();
	}
	g_iSky = GetRandomInt(0, g_aSky.Length - 1);
	
	g_aSky.GetString(g_iSky, g_sSkyBox, sizeof(g_sSkyBox));
	
	if( IsDefaultSkybox(g_sSkyBox) )
	{
		// is Sacrifice?
		if( strcmp(g_sMap, "l4d_river01_docks") == 0
		||	strcmp(g_sMap, "l4d_river02_barge") == 0
		||	strcmp(g_sMap, "l4d_river03_port") == 0 )
		{
			OnMapStart();
			return;
		}
	}
	
	DownloadSkyboxes();
	
	SetSkyname(g_sSkyBox);
	
	CreateTimer(30.0, Timer_DeafultSky, _, TIMER_FLAG_NO_MAPCHANGE);
}

Action Timer_DeafultSky(Handle timer)
{
	CreateTimer(5.0, Timer_WaitConnection, _, TIMER_FLAG_NO_MAPCHANGE | TIMER_REPEAT);
}

Action Timer_WaitConnection(Handle timer)
{
	for( int i = 1; i <= MaxClients; i++ )
	{
		if (IsClientConnected(i) && !IsClientInGame(i))
			return Plugin_Continue;
			
		if (IsClientInGame(i) && GetClientTeam(i) == 0)
			return Plugin_Continue;
	}
	
	if( g_sSkyBoxDefault[0] )
	{
		g_hCvarSkyName.SetString(g_sSkyBoxDefault);
	}
	return Plugin_Stop;
}

void DownloadSkyboxes()
{
	if( g_sSkyBox[0] != '\0' )
	{
		if ( !IsDefaultSkybox(g_sSkyBox) )
		{
			DownloadSkybox(g_sSkyBox);
		}
	}
}

void GetSkyBoxes()
{
	int iLen;
	char sSky[64];
	FileType fileType;
	DirectoryListing hDir;
	
	hDir = OpenDirectory("materials/skybox", true, NULL_STRING); // virtual + real
	if( hDir )
	{
		while( hDir.GetNext(sSky, sizeof sSky, fileType))
		{
			if( fileType == FileType_File )
			{
				iLen = strlen(sSky);
				
				if( iLen > 6 )
				{
					sSky[iLen - 6] = '\0';
					hMapSky.SetValue(sSky, 1);
				}
			}
		}
		delete hDir;
	}
	
	hDir = OpenDirectory("materials/skybox", false, NULL_STRING); // -remove real
	if( hDir )
	{
		while( hDir.GetNext(sSky, sizeof sSky, fileType))
		{
			if( fileType == FileType_File )
			{
				iLen = strlen(sSky);
				
				if( iLen > 6 )
				{
					sSky[iLen - 6] = '\0';
					hMapSky.Remove(sSky);
				}
			}
		}
		delete hDir;
	}
	
	/*
	StringMapSnapshot hSnap = hMapSky.Snapshot();
	
	for (int i = 0; i < hSnap.Length; i++ )
	{
		hSnap.GetKey(i, sSky, sizeof sSky);
		PrintToServer(sSky);
	}
	delete hSnap;
	*/
}

stock bool IsDefaultSkybox(char[] sSky)
{
	int v;
	return hMapSky.GetValue(sSky, v);
}

void DownloadSkybox(char[] sSkybox)
{
	static char sPath[PLATFORM_MAX_PATH];
	char Pref[][] = {"bk", "Bk", "dn", "Dn", "ft", "Ft", "lf", "Lf", "rt", "Rt", "up", "Up" };
	for( int i = 0; i < sizeof(Pref); i++ )
	{
		FormatEx(sPath, sizeof(sPath), "materials/skybox/%s%s.vmt", sSkybox, Pref[i]);
		if (FileExists(sPath)) AddFileToDownloadsTable(sPath);
		FormatEx(sPath, sizeof(sPath), "materials/skybox/%s%s.vtf", sSkybox, Pref[i]);
		if (FileExists(sPath)) AddFileToDownloadsTable(sPath);
	}
}

void SetSkyname(char[] sSky)
{
	if( sSky[0] != '\0' )
	{
		g_hCvarSkyName.SetString(sSky);
	}
}

stock bool IsClientTransitional()
{
	
}